from setuptools import find_packages, setup


def get_version(filename):
    from re import findall
    with open(filename) as f:
        metadata = dict(findall("__([a-z]+)__ = '([^']+)'", f.read()))
    return metadata['version']

setup(
    name='cachetools',
    version=get_version('cachetools/__init__.py'),
    url='https://github.com/tkem/cachetools',
    license='MIT',
    author='Thomas Kemmer',
    author_email='tkemmer@computer.org',
    description='Extensible memoizing collections and decorators',
    long_description=open('README.rst').read(),
    keywords='cache caching memoize memoizing memoization LRU LFU TTL',
    packages=find_packages(exclude=['tests', 'tests.*']),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)
